unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ComCtrls, ExtCtrls, ImgList, Menus, ActnList;

type
  TwndMain = class(TForm)
    TreeView1: TTreeView;
    Splitter1: TSplitter;
    ListView1: TListView;
    MainMenu1: TMainMenu;
    ActionList1: TActionList;
    acSave: TAction;
    Configuration1: TMenuItem;
    Saveastext1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    ImageList1: TImageList;
    SaveDialog1: TSaveDialog;
    acChangeValue: TAction;
    Changevalue1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure TreeView1Change(Sender: TObject; Node: TTreeNode);
    procedure ListView1Editing(Sender: TObject; Item: TListItem;
      var AllowEdit: Boolean);
    procedure TreeView1Editing(Sender: TObject; Node: TTreeNode;
      var AllowEdit: Boolean);
    procedure Exit1Click(Sender: TObject);
    procedure acSaveExecute(Sender: TObject);
    procedure acChangeValueExecute(Sender: TObject);
    procedure acChangeValueUpdate(Sender: TObject);
    procedure ListView1DblClick(Sender: TObject);
  private
    procedure FillTree(Node: TTreeNode; const Path: string);
  public
  end;

var
  wndMain: TwndMain;

implementation

uses BDE, Change;

{$R *.DFM}

procedure Check(Error: Integer);
begin
  if Error <> 0 then
    raise Exception.CreateFmt('BDE Error #%.4x', [Error]);
end;

procedure SetBDEInfo(const Path, Param, Value: string);
var
  hCur: HDbiCur;
  Desc: CFGDesc;
begin
  Check(DbiOpenCfgInfoList(nil, dbiReadWrite, cfgPersistent, PChar(Path), hCur));
  try
    while DbiGetNextRecord(hCur, dbiNoLock, @Desc, nil) = DBIERR_NONE do
      if StrIComp(Desc.szNodeName, PChar(Param)) = 0 then
      begin
        StrCopy(Desc.szValue, PChar(Value));
        DbiModifyRecord(hCur, @Desc, True);
        Break;
      end;
  finally
    DbiCloseCursor(hCur);
  end;
end;

function NodePath(T: TTreeNode): string;
begin
  Result := '';
  repeat
    Result := '\' + Result;
    if T.Level <> 0 then Result := T.Text + Result;
    T := T.Parent;
  until T = nil;
end;

procedure TwndMain.FillTree(Node: TTreeNode; const Path: string);
var
  hCur: HDbiCur;
  Desc: CFGDesc;
begin
  Check(DbiOpenCfgInfoList(nil, dbiReadOnly, cfgPersistent, PChar(Path), hCur));
  try
    while DbiGetNextRecord(hCur, dbiNoLock, @Desc, nil) = DBIERR_NONE do
      if Desc.bHasSubNodes then
      begin
        FillTree(TreeView1.Items.AddChild(Node, StrPas(Desc.szNodeName)),
          Path + StrPas(Desc.szNodeName) + '\');
      end;
  finally
    DbiCloseCursor(hCur);
  end;
end;

procedure TwndMain.FormCreate(Sender: TObject);
begin
  Check(DbiInit(nil));
  FillTree(TreeView1.Items.AddChild(nil, 'BDE'), '\');
end;

procedure TwndMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  DbiExit;
end;

procedure TwndMain.TreeView1Change(Sender: TObject; Node: TTreeNode);
var
  hCur: HDbiCur;
  Desc: CFGDesc;
begin
  ListView1.Items.BeginUpdate;
  try
    ListView1.Items.Clear;
    Check(DbiOpenCfgInfoList(nil, dbiReadOnly, cfgPersistent,
      PChar(NodePath(Node)), hCur));
    try
      while DbiGetNextRecord(hCur, dbiNoLock, @Desc, nil) = DBIERR_NONE do
        if not Desc.bHasSubNodes then
          with ListView1.Items.Add do
          begin
            Caption := StrPas(Desc.szNodeName);
            SubItems.Add(StrPas(Desc.szValue));
          end;
    finally
      DbiCloseCursor(hCur);
    end;
  finally
    ListView1.Items.EndUpdate;
  end;
end;

procedure TwndMain.ListView1Editing(Sender: TObject; Item: TListItem;
  var AllowEdit: Boolean);
begin
  AllowEdit := False;
end;

procedure TwndMain.TreeView1Editing(Sender: TObject; Node: TTreeNode;
  var AllowEdit: Boolean);
begin
  AllowEdit := False;
end;

procedure TwndMain.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TwndMain.acSaveExecute(Sender: TObject);
var
  Txt: TextFile;

  procedure VisitNode(const APath: string);
  var
    hCur: HDbiCur;
    Desc: CFGDesc;
    FirstTime: Boolean;
  begin
    Check(DbiOpenCfgInfoList(nil, dbiReadOnly, cfgPersistent, PChar(APath + '\'), hCur));
    try
      FirstTime := True;
      while DbiGetNextRecord(hCur, dbiNoLock, @Desc, nil) = DBIERR_NONE do
        if not Desc.bHasSubNodes then
        begin
          if FirstTime then
            WriteLn(Txt, '[' + APath + ']');
          FirstTime := False;
          WriteLn(Txt, StrPas(Desc.szNodeName), '=', StrPas(Desc.szValue));
        end;
      if not FirstTime then WriteLn(Txt);
      DbiSetToBegin(hCur);
      while DbiGetNextRecord(hCur, dbiNoLock, @Desc, nil) = DBIERR_NONE do
        if Desc.bHasSubNodes then
          VisitNode(APath + '\' + StrPas(Desc.szNodeName));
    finally
      DbiCloseCursor(hCur);
    end;
  end;

begin
  if SaveDialog1.Execute then
  begin
    AssignFile(Txt, SaveDialog1.FileName);
    Rewrite(Txt);
    try
      VisitNode('');
    finally
      CloseFile(Txt);
    end;
  end;
end;

procedure TwndMain.acChangeValueExecute(Sender: TObject);
begin
  with TwndChange.Create(nil) do
  try
    Edit1.Text := NodePath(TreeView1.Selected);
    Edit2.Text := ListView1.Selected.Caption;
    Edit3.Text := ListView1.Selected.SubItems[0];
    if ShowModal = mrOk then
    begin
      SetBDEInfo(Edit1.Text, Edit2.Text, Edit3.Text);
      ListView1.Selected.SubItems[0] := Edit3.Text;
    end;
  finally
    Free;
  end;
end;

procedure TwndMain.acChangeValueUpdate(Sender: TObject);
begin
  acChangeValue.Enabled := Assigned(TreeView1.Selected)
    and Assigned(ListView1.Selected);
end;

procedure TwndMain.ListView1DblClick(Sender: TObject);
begin
  acChangeValue.Execute;
end;

end.
